from flask import Flask, request, jsonify
import database_helper as dbh
import json

app=Flask(__name__)


## Helper function ------------------------------------------------
def get_data(email):
    user = dbh.find_user(email)
    if(user == None):
        return jsonify({"success":False,"message":"User associated with email doesn't exist!","data":"-"})

    returnData = {'email':user['email'],'firstname':user['firstname'],'familyname':user['familyname'],'gender':user['gender'],'city':user['city'],'country':user['country']}

    return jsonify({"success":True,"message":"Fetched user data successfully!","data":returnData})

def get_messages(email):
    user = dbh.find_user(email)
    if(user == None):
        return jsonify({"success":False,"message":"User associated with email doesn't exist!","data":"-"})
    messages = dbh.retrieve_messages(email)

    return jsonify({"success":True,"message":"Existing messages fetched!","data":messages})
    return
## ----------------------------------------------------------------


@app.route('/')
def hello():
    return 'Hello!'

@app.route('/sign_in',methods=['POST'])
def sign_in():
    data = request.get_json()

    email    = data['email']
    password = data['password']

    user = dbh.find_user(email)

    if(user == None):
        return jsonify({"success":False,"message":"No such user exist","data":"-"})

    if(user['password'] == password):
        token = dbh.new_token(email)
        return jsonify({"success":True,"message":"Login successful","data":token})
    else:
        return jsonify({"success":False,"message":"Incorrect password","data":"-"})


@app.route('/sign_out', methods=['DELETE'])
def sign_out():
    token= request.headers.get("Authorization")

    email = dbh.get_online_user(token)

    if(email == None):
        return jsonify({"success":False,"message":"Invalid access token!","data":"-"})

    result=dbh.clear_token(token)
    if(result == True):
        return jsonify({"success":True,"message":"Logged out successfully!","data":"-"})
    else:
        return jsonify({"success":False,"message":"Error encountered while trying to delete token!","data":"-"})



@app.route('/sign_up',methods=['POST'])
def sign_up():
    #return "",200
    data = request.get_json()
    # Retrieve Data
    email      =data['email']
    password   =data['password']
    firstname  =data['firstname']
    familyname =data['familyname']
    gender     =data['gender']
    city       =data['city']
    country    =data['country']


    # Valdiating inputs
    if(len(email) == 0 or email==None):
        return jsonify({"success":False,"message":"The email field needs to be filled out!","data":"-"})
    if(len(password) < 5):
        return jsonify({"success":False,"message":"The password needs to consist of 5 or more character!","data":"-"})
    if(len(firstname) == 0 or firstname==None):
        return jsonify({"success":False,"message":"The first name field needs to be filled out!","data":"-"})
    if(len(familyname) == 0 or familyname==None):
        return jsonify({"success":False,"message":"The family name field needs to be filled out!","data":"-"})
    if(len(gender) == 0 or gender==None):
        return jsonify({"success":False,"message":"The gender field needs to be filled out!","data":"-"})
    if(len(city) == 0 or city==None):
        return jsonify({"success":False,"message":"The city field needs to be filled out!","data":"-"})
    if(len(country) == 0 or country==None):
        return jsonify({"success":False,"message":"The country field needs to be filled out!","data":"-"})

    user = dbh.find_user(email)

    if(user == None):
        result = dbh.new_user(email,password,firstname,familyname,gender,city,country)
        if(result == True):
            return jsonify({"success":True,"message":"Account created successfuly","data":"-"})
        else:
            return jsonify({"success":False,"message":"Error while creating table entry","data":"-"})
    else:
        return jsonify({"success":False,"message":"User already exist!","data":"-"})

@app.route('/change_password')
def change_password():
    token=request.headers.get("Authorization")
    oldPassword="Asdasdada"
    newPassword="cvxfsfdsfd"
    email=dbh.get_online_user(token)
    user=find_user(email)
    if(user == None):
        return jsonify({"success":False,"message":"Account associated with token was not found!","data":"-"})

    if(user['password'] != oldPassword):
        return jsonify({"success":False,"message":"The old password is incorrect!","data":"-"})

    result = new_password(email,newPassword);

    if(result == True):
        return jsonify({"success":True,"message":"Your password has been updated!","data":"-"})
    else:
        return jsonify({"success":False,"message":"Something went wrong while trying to change the password!","data":"-"})


@app.route('/get_user_data_by_token', methods=['GET'])
def get_user_data_by_token():
    token=request.headers.get("Authorization")
    email=dbh.get_online_user(token)
    return get_data(email)


@app.route('/get_user_data_by_email/<email>', methods=['GET'])
def get_user_data_by_email(email=""):
    return get_data(email)


@app.route('/get_user_messages_by_email/<email>', methods=['GET'])
def get_user_messages_by_email(email=""):
    return get_messages(email)

@app.route('/get_user_messages_by_token')
def get_user_messages_by_token():
    token=request.headers.get("Authorization")
    email=dbh.get_online_user(token)
    return get_messages(email)

@app.route('/post_message', methods=['PUT'])
def post_message():
    token = request.headers.get("Authorization")
    data = request.get_json()
    # Retrieve Data
    fromEmail = dbh.get_online_user(token)
    if(fromEmail == None):
        return jsonify({"success":False,"message":"Account associated with token was not found!","data":"-"})
    toEmail      =data['to']
    if(dbh.find_user(toEmail) == None):
        return jsonify({"success":False,"message":"Can't create a post to a user that doesn't exist!","data":"-"})
    message      =data['message']

    result = dbh.create_post(toEmail,fromEmail,message)

    if(result == True):
        return jsonify({"success":True,"message":"Post created!","data":"-"})
    else:
        return jsonify({"success":False,"message":"Post didn't create!","data":"-"})


# Main function =========================================================================
if __name__ == '__main__':
    app.debug = True #Comment this line while not debugging
    app.run()
